#pragma once

#include "Universe.hpp"
#include "../net/Client.hpp"
#include "core/utils/zctx.hpp"
#include "core/world/Elements.hpp"

namespace world::client {
    class Area;

    /// Whole universe container in network client
    class DistantUniverse final: public Universe {
    public:
        DistantUniverse(const login&, const connection&, const options &, const std::string&);
        ~DistantUniverse();

        void update(voxel_pos, float) override;
        void emit(const action::packet &) override;

        bool isRunning() const override { return peer.isRunning(); }

        world::node_id getPlayerId() const override { return playerId; }
        virtual mutex::abstract_shared_guard<Elements>::handle getElements() const override { return mutex::not_guard<Elements>::OfShared(&elements); };

    protected:
        class MyElements final: public world::EdittableElements {
            public:
                MyElements();

                node_id createAt(node_id, const start_point&, const std::shared_ptr<world::Element>&);
        };

        constexpr uint16_t getQueryDistance() const { return std::min(options.loadDistance, serverDistance); }

        bool onPacket(const memory::read_view&, net::PacketFlags);

        /// Universe client size view
        MyElements elements;

        /// Player entity instance index
        world::node_id playerId;
        std::optional<zstd::read_dict_ctx> dict;

        bool mayQueryChunks = false;
        uint64_t moveCounter = 0;

        options options;
        uint16_t serverDistance;
        size_t floodfillLimit = CHUNK_SIZE;

        net::client::Client peer;
    };
}