#pragma once
#include <array>

/// Generic containers
namespace data {
    /// Looping array
    template <typename T, std::size_t N>
    struct circular_buffer: public std::array<T, N> {
        using buffer = std::array<T, N>;

        circular_buffer(): buffer() { }
        circular_buffer(const T& zero): buffer() {
            buffer::fill(zero);
        }

        size_t next = 0;

        void move_back() {
            next++;
            if (next >= buffer::size())
                next -= buffer::size();
        }
        void push_back(const T& in) {
            buffer::at(next) = in;
            move_back();
        }

        template<typename F>
        void iter(F f) const {
            for (size_t i = next; i < buffer::size(); i++) {
                f(buffer::at(i));
            }
            for (size_t i = 0; i < next; i++) {
                f(buffer::at(i));
            }
        }
    };
}
