#include "Elements.hpp"

using namespace world;

transform Elements::computeAbsolute(const node_id &pid, const transform& rel) const {
    //FIXME: include angle
    if (auto parent = nodes.directly_at(pid)) {
        return parent->absolute.computeChild(rel);
    }
    return rel;
}
world_pos Elements::computeAbsolute(const node_ref &pref, const voxel_pos& rel) const {
    if (auto parent = nodes.directly_at(pref)) {
        return parent->absolute.computeChild(rel);
    }
    return rel;
}

node_id Elements::findParent(const node_id& id) const {
    if (const auto it = hierarchy.get(id)) {
        return it->parent;
    }
    return generational::id();
}

EdittableElements::EdittableElements(bool loadModels) {
    if (loadModels) {
        module::Registry::Get()->extractModels([&](module::registered_model& model) {
            const auto res = models.emplace(0, std::move(model.dt), model.collision, model.scale);
            res.second->origin = std::make_shared<Instance>(res.first);
        });
    }
}

void EdittableElements::remove(node_id i) {
    auto id = nodes.with_flag(i);
    extract(id.val.index());
    nodes.at(id).content->free(id, *this);
    nodes.free(id);
    nodes.set_flag(Set<Flag::Removed>(id));
}
void EdittableElements::removeRecursive(node_id i) {
    hierarchy.erase_recursive(i.val.index(), [&](generational::hierarchy<Node, Hierarchy>::entry *entry) {
        auto id = nodes.with_flag(entry->self);
        nodes.at(id).content->free(id, *this);
        nodes.free(id);
        nodes.set_flag(Set<Flag::Removed>(id));
    });
}

void EdittableElements::move(node_id i, const relative_pos &p) {
    const auto idx = i.val.index();
    auto it = hierarchy.get(idx);
    assert(it);
    if (it->parent != p.parent) {
        //FIXME: keep velocity and rotation (make absolute)
        extract(idx); //FIXME: move recursive
        insert(idx, p);
    } else if (it->relative.position != p.position) {
        nodes.set_flag(Set<Flag::Moved>(nodes.with_flag(i)));
        it->relative.position = p.position;
    }
    nodes.at(i).absolute = computeAbsolute(p.parent, it->relative);
}
void EdittableElements::move(node_id i, const start_point &p) {
    const auto idx = i.val.index();
    auto it = hierarchy.get(idx);
    assert(it);
    if (it->parent != p.parent) {
        extract(idx); //FIXME: move recursive
        insert(idx, p);
    } else if (it->relative != p.relative || it->vel != p.vel) {
        nodes.set_flag(Set<Flag::Moved>(nodes.with_flag(i)));
        it->relative = p.relative;
        it->vel = p.vel;
    }
    nodes.at(i).absolute = computeAbsolute(p.parent, it->relative);
}

void EdittableElements::extract(node_ref r) {
    hierarchy.erase(r, [&](generational::hierarchy<Node, Hierarchy>::entry *child) {
        nodes.set_flag(Set<Flag::Moved>(nodes.with_flag(child->self)));
    });
}
void EdittableElements::insert(node_ref r, const start_point& sp) {
    hierarchy.set_emplace(r, sp.parent, sp.relative, sp.vel);
}

world::node_id EdittableElements::create(const start_point& sp, uint8_t flag, const std::shared_ptr<world::Element>& el) {
    const auto id = nodes.emplace(flag, computeAbsolute(sp.parent, sp.relative), el).first;
    assert(Has<Flag::Added>(id));
    if (!Has<Flag::Fixed>(id))
        insert(id, sp);
    return id;
}
world::node_id EdittableElements::createAt(node_id i, const start_point& sp, const std::shared_ptr<world::Element>& el) {
    const auto id = Set<Flag::Added>(i);
    nodes.set_emplace(id, computeAbsolute(sp.parent, sp.relative), el);
    if (!Has<Flag::Fixed>(id))
        insert(id, sp);
    return id;
}
world::node_id EdittableElements::createAt(generational::id::idx_t i, const start_point& sp, uint8_t flag, const std::shared_ptr<world::Element>& el) {
    return createAt(generational::id(i, 0, flag), sp, el);
}